import 'dart:io';
import 'dart:async';
import 'package:wrapper_args/wrapper_args.dart';

Command init() {
  final parser = ArgParser()
    ..addFlag(
      "help",
      abbr: "h",
      negatable: false,
      help: "Print this usage information",
    )
    ..addOption(
      "output",
      abbr: "o",
    );
  return Command(
    name: "bash-completion",
    help: "generate bash-completion for bash shell",
    parser: parser,
    execute: (command, results) {
      if (results["help"]) {
        print(command.usage);
      } else {
        final controller = StreamController<int>();
        controller.stream.listen((v) {
          exitCode = v ?? 0;
        });
        () async {
          final output = results["output"];
          IOSink writer;
          if (output == null) {
            writer = stdout;
          } else {
            var file = File(output);
            file = await file.create(recursive: false);
            writer = file.openWrite();
          }
          final now = DateTime.now();
          writer.write('''#!/bin/bash
#Program:
#       generate for bash completion
#History:
#       tools auto create at $now
#Email:
#       zuiwuchang@gmail.com

''');
          writer.write(r'''
function __king011_generate_bash_completion(){
  local opts='-h --help \
    -o --output'

  case ${COMP_WORDS[COMP_CWORD-1]} in
    -o|--output)
      _filedir || COMPREPLY=( $(compgen -o plusdirs -f ${cur}) )
    ;;

    *)
      COMPREPLY=( $(compgen -W "${opts}" -- ${cur}) )
    ;;
  esac  
}
function __king011_generate_dart_version(){
  local opts='-h --help \
    -o --output'

  case ${COMP_WORDS[COMP_CWORD-1]} in
    -o|--output)
      _filedir || COMPREPLY=( $(compgen -o plusdirs -f ${cur}) )
    ;;

    *)
      COMPREPLY=( $(compgen -W "${opts}" -- ${cur}) )
    ;;
  esac
}
function __king011_generate_dart(){
  if [ 2 == $COMP_CWORD ];then
    local opts='-h --help \
      version'
    COMPREPLY=( $(compgen -W "${opts}" -- ${cur}) )
  else
    case ${COMP_WORDS[2]} in

      version)
      __king011_generate_dart_version
      ;;

    esac
  fi
}

function __king011_generate_go_version(){
  local opts='-h --help \
    -o --output'

  case ${COMP_WORDS[COMP_CWORD-1]} in
    -o|--output)
      _filedir || COMPREPLY=( $(compgen -o plusdirs -f ${cur}) )
    ;;

    *)
      COMPREPLY=( $(compgen -W "${opts}" -- ${cur}) )
    ;;
  esac
}

function __king011_generate_go_init(){
  local opts='-h --help \
    -o --output --template --package-name --project-name'

  case ${COMP_WORDS[COMP_CWORD-1]} in
    -o|--output)
      _filedir || COMPREPLY=( $(compgen -o plusdirs -f ${cur}) )
    ;;

    --template)
      COMPREPLY=( $(compgen -W "console" -- ${cur}) )
    ;;

    *)
      COMPREPLY=( $(compgen -W "${opts}" -- ${cur}) )
    ;;
  esac
}

function __king011_generate_go(){
  if [ 2 == $COMP_CWORD ];then
    local opts='-h --help \
      init version'
    COMPREPLY=( $(compgen -W "${opts}" -- ${cur}) )
  else
    case ${COMP_WORDS[2]} in

      init)
        __king011_generate_go_init
      ;;

      version)
        __king011_generate_go_version
      ;;
    esac
  fi
}
function __king011_generate(){
	local cur=${COMP_WORDS[COMP_CWORD]}
	if [ 1 == $COMP_CWORD ];then
		local opts='-h --help -v --version \
			bash-completion \
			dart'
		COMPREPLY=( $(compgen -W "${opts}" -- ${cur}) )
	else
		case ${COMP_WORDS[1]} in

			bash-completion)
				__king011_generate_bash_completion
			;;

			dart)
				__king011_generate_dart
			;;

			go)
				__king011_generate_go
			;;
		esac
	fi
}

complete -F __king011_generate generate
''');

          if (output != null) {
            await writer.close();
          }
        }();
      }
    },
  );
}
