import 'dart:io';
import 'package:wrapper_args/wrapper_args.dart';
import '../utils/version.dart';

Command init() {
  final parser = ArgParser()
    ..addFlag(
      "help",
      abbr: "h",
      negatable: false,
      help: "Print this usage information",
    );
  return Command(
    name: "dart",
    help: "generate dart code",
    longHelp: "Usage: generate dart [<subcommand>] [<options>]",
    parser: parser,
    execute: (command, results) {
      if (results["help"]) {
        print(command.usage);
      }
    },
  )..addCommand(
      [
        _version(),
      ],
    );
}

Command _version() {
  final parser = ArgParser()
    ..addFlag(
      "help",
      abbr: "h",
      negatable: false,
      help: "Print this usage information",
    )
    ..addOption(
      "control",
      defaultsTo: Version.defaultsTo,
      allowed: Version.allowed,
      help: "version control",
    )
    ..addOption(
      "output",
      abbr: "o",
      defaultsTo: "lib/version.dart",
    );
  return Command(
    name: "version",
    help: "generate dart version code from source",
    longHelp: "Usage: generate dart version [<options>]",
    parser: parser,
    execute: (command, results) {
      if (results["help"]) {
        print("${command.usage}");
        return 0;
      }
      () async {
        final stat = await FileStat.stat("pubspec.yaml");
        if (stat.type == FileSystemEntityType.notFound) {
          throw StateError("pubspec.yaml not found");
        }
        final output = results["output"];
        final version = Version.fromControl(results["control"]);

        print('''
tag      = ${version.tag} 
commit   = ${version.commit}
datetime = ${version.datetime}
''');

        var file = File(output);
        file = await file.create(recursive: false);
        final writer = file.openWrite();
        writer.write('''/// version create by auto generate tools
class Version{
  Version._();
  static String get tag => '${version.tag}';
  static String get commit => '${version.commit}';
  static String get datetime => '${version.datetime}';
} 
''');
        await writer.close();
      }();
    },
  );
}
